/**
 * @file
 * Adds JS functionality to the Private Message Windows module.
 */

/*global jQuery, Drupal, drupalSettings, window*/

(function ($, Drupal, drupalSettings, window) {
  Drupal.PrivateMessageInbox = Drupal.PrivateMessageInbox || {};
//  Drupal.PrivateMessageInbox.updateInbox = Drupal.PrivateMessageInbox.updateInbox || {};

  "use strict";

  var pmWindows, updateInterval, inboxRequesting, blockSettings;
  var requesting = { 'initedNeed': 1, 'thread': {}, 'threadRefresh': {}, 'msgSend': {}, 'newThread': 0 };
  var threadLoading = {};
  var winWidth = 260;

  /**
   * Used to manually trigger Drupal's JavaScript commands.
   * @param {Object} data The data.
   */
  function triggerCommands(data) {
    var ajaxObject = Drupal.ajax({
      url: '',
      base: false,
      element: false,
      progress: false
    });

    // Trigger any ajax commands in the response.
    ajaxObject.success(data, 'success');
  }

  /**
   * Initializes the private message inbox window.
   */
  function init(context) {
    if(!requesting.initedNeed) return;

    var block = $('.block-private-message-windows-block');
    if(!block.length) { // block is still absent in the site DOM
      requesting.initedNeed++;
      setTimeout(function(){ init(context); }, 300);
      return;
    }
    requesting.initedNeed = 0;

    block.once('overlay-block-ajax').each(function(){
      //console.log('drupalSettings', drupalSettings);
      if(!drupalSettings.privateMessageWindowsBlock) return;

      blockSettings = drupalSettings.privateMessageWindowsBlock;

      pmWindows = $(this).first().find('.content:first');
      pmWindows.append('<style>.block-private-message-windows-block .label { color: ' +
        blockSettings.titleTextColor
        + '; background-color: '+
        blockSettings.titleTextBackground
        + '; } .block-private-message-windows-block .content.new .label.main-title {' +
        '  color: ' + blockSettings.titleNewMsgTextColor + ';' +
        '} </style>');

      updateInterval = blockSettings.ajaxRefreshRate * 1000;
      if (updateInterval) {
        window.setTimeout(getUpdateInbox, updateInterval);
      }

      pmWindows.find('.inbox-thread-list .thread').on('click', inboxThreadLinkListenerHandler);
      pmWindows.find('.label').on('click', showHideWin);
      $('a[href*="/private-message/create?recipient="]').on('click', newRecipientMessage);
    });
  }


  /**
   * Show and hide inbox or thread window
   */
  function showHideWin() {
    $(this).parent().toggleClass('active');
  }

  /**
   * Run update the inbox.
   */
  function getUpdateInbox() {
    if (!inboxRequesting) {
      inboxRequesting = true;

      var ids = {};
      pmWindows.find('.inbox-thread-list .thread').each(function () {
        ids[$(this).attr('data-thread-id')] = $(this).attr('data-last-update');
      });

      $.ajax({
        url: blockSettings.loadNewThreadUrl,
        method: "POST",
        data: { ids: ids },
        success: function (data) {
          inboxRequesting = false;
          triggerCommands(data);
          if (updateInterval) {
            window.setTimeout(getUpdateInbox, updateInterval);
          }
        }
      });
    }
  }


  /**
   * Refresh inbox after ajax call
   * @param threadIds - ids of threads
   * @param newThreads - updated threads
   */
  function updateInbox(threadIds, newThreads) {
    var revertedIndex = [];
    for(var index in threadIds) {
      revertedIndex.unshift(index);
    }

    var inboxWrapper = pmWindows.find('.inbox-thread-list');
    if(threadIds.length) inboxWrapper.find('.no-thread').remove();

    for(var i in revertedIndex) {
      var index = revertedIndex[i];
      var threadId = threadIds[index];
      var message = inboxWrapper.find('.thread-'+threadId);

      if(message.length) {
        message.remove().empty();
        refreshThread(threadId);
      }
      inboxWrapper.prepend(newThreads[threadId]);
      var newThread = inboxWrapper.find('.thread-'+threadId);
      if(!newThread.hasClass('own')) {
        inboxWrapper.find('.thread-'+threadId).addClass('new');
        pmWindows.addClass('new')
      }

      inboxWrapper.find('.thread-'+threadId).on('click', inboxThreadLinkListenerHandler);
    };
  }


  /**
   * Click Handler executed when private message threads are clicked.
   *
   * Loads the thread into the private message window.
   */
  var inboxThreadLinkListenerHandler = function (e) {
    e.preventDefault();
    var threadId = $(this).attr("data-thread-id");
    var inboxThread = pmWindows.find('.inbox-thread-list .thread-'+threadId);
    var thWrapper = pmWindows.find('.threads-wrapper');
    var thread =  thWrapper.find('.thread-'+threadId);
    if (thread.length) {
      thread.addClass('active');
      if(inboxThread.hasClass('new')) {
        inboxThread.removeClass('new');
        if(!pmWindows.find('.threads-wrapper .thread.new').length) {
          pmWindows.removeClass('new');
        }
        thread.find('textarea').focus();
      }else {
        if(thread.is(':hidden')) {
          hideLastWin(thWrapper);
        }
        thread.toggle();
      }
    } else {
      inboxThread.removeClass('new');
      loadThread(threadId);
    }
  };


  /**
   * Downloading thread content
   * @param threadId
   */
  function loadThread(threadId) {
    var threadDiv = pmWindows.find('.thread-wrapper .thread-'+threadId);
    if(threadDiv.length) return;

    if(requesting.thread[threadId]) return;
    requesting.thread[threadId] = true;

    $.ajax({
      url: blockSettings.loadThreadMsgsUrl,
      data: { threadid:threadId },
      success: function (data) {
        requesting.thread[threadId] = false;
        triggerCommands(data);
      }
    });
  }

  function hideLastWin(thWrapper) {
    var thWrapperW = thWrapper.width();
    var winLimit = Math.floor(thWrapperW/winWidth);
    var existsThreads = thWrapper.find('.thread:visible');
    if(existsThreads.length >= winLimit) {
      existsThreads.each(function(i,item) {
        if((i-(-1)) >= winLimit) $(item).hide();
      });
    }

  }

  /**
   * Filling thread after ajax request
   * @param threadId - thread id
   * @param userName - name of user
   * @param userPicture - image url of user icon
   * @param messages - html of thread messages
   */
  function loadThreadResponse(threadId, lastMsgId, userName, userPicture, messages) {
    var thWrapper = pmWindows.find('.threads-wrapper');
    hideLastWin(thWrapper);

    thWrapper.prepend('<div class="thread thread-' + threadId + ' active" data-thread-id="' +
      threadId + '" data-last-msg-id="'+lastMsgId+'">' +
      '<div class="label"><div class="x">&#xd7;</div><img src="' + userPicture + '" />' + userName + '</div>' +
      '<div class="messages"></div><div class="operations"><textarea></textarea><input type="button" value="Send" /></div></div>');
    var threadDiv = thWrapper.find('.thread-'+threadId);
    var threadMsgDiv = threadDiv.find('.messages');
    threadMsgDiv.prepend(messages);
    threadDiv.find('input[type="button"]').on('click', sendMessage);
    threadDiv.find('.label').on('click', showHideWin);
    threadDiv.find('.label .x').on('click', function () { $(this).closest('.thread').hide(); return false; });
    messagesScrollDown(threadDiv);
  }


  /**
   * Send message to user
   */
  function sendMessage() {
    var $this = $(this);
    var operations = $this.closest('.operations');
    operations.addClass('sending');
    var textArea = operations.find('textarea');
    var text = textArea.val();
    var thread = $this.closest('.thread');
    var threadId = thread.attr('data-thread-id');
    var msgId = thread.attr('data-last-msg-id');

    if(requesting.msgSend[threadId]) return;
    requesting.msgSend[threadId] = true;

    $.ajax({
      url: blockSettings.sendMsgUrl,
      data: { thread_id:threadId, text:text, last_message_id:msgId },
      method: 'POST',
      success: function (data) {
        textArea.val('');
        requesting.msgSend[threadId] = false;
        operations.removeClass('sending');
        triggerCommands(data);
      }
    });

  }


  /**
   * Response to ajax request of message sending or thread update
   */
  function updateThreadMsgs(newMessages, threadId, lastMsgId) {
    var thread = pmWindows.find('.threads-wrapper .thread-'+threadId);
    var threadMessages = thread.find('.messages');
    //to avoid duplicate of messages, we add each message with existing check
    $('<div>' + newMessages + '</div').find('.private-message').each(function(i, item) {
      var msg = $(item);
      var msgId = msg.attr('data-message-id');
      if(!threadMessages.find('#private-message-'+msgId).length) {
        threadMessages.append(item);
        var oldMsgId = thread.attr('data-last-msg-id');
        if (oldMsgId < msgId) thread.attr('data-last-msg-id', msgId);
      }
    });
    messagesScrollDown(thread);
    getUpdateInbox();
  }


  /**
   * Refresh of thread window for new messages
   * @param threadId
   */
  function refreshThread(threadId) {
    var threadDiv = pmWindows.find('.threads-wrapper .thread-'+threadId);
    if(!threadDiv.length) return;

    if(requesting.threadRefresh[threadId]) return;
    requesting.threadRefresh[threadId] = true;

    var msgId = threadDiv.attr('data-last-msg-id');
    $.ajax({
      url: blockSettings.loadNewMsgsUrl,
      data: { thread_id:threadId, message_id: msgId },
      success: function (data) {
        requesting.threadRefresh[threadId] = false;
        triggerCommands(data);
      }
    });
  }

  /**
   * Moving message list to the last message
   * @param threadDiv
   */
  function messagesScrollDown(threadDiv) {
    var messages = threadDiv.find('.messages');
    var height = 0;
    messages.find('.private-message').each(function(i,item){
      height -= (-1)*$(item).height();
    });
    messages.scrollTop(height - (-1)*300);
  }


  function newRecipientMessage(e) {
    e.stopPropagation();
    e.preventDefault();
    if(requesting.newThread) return;
    requesting.newThread = true;

    var aTag = $(this);
    var href = aTag.attr('href');
    var check = new RegExp('recipient=(\\d+)');
    var checkRes = check.exec(href);
    if(!checkRes[1] || !(checkRes[1]>0)) {
      window.location = href;
      return;
    }
    var recipientId = checkRes[1];

    $.ajax({
      url: blockSettings.newThreadUrl,
      data: { recip_id:recipientId },
      method: 'POST',
      success: function (data) {
        requesting.newThread = false;
        triggerCommands(data);
      }
    });

  }


  Drupal.behaviors.privateMessageWindowsThreadList = {
    attach:function (context) {
      init(context);

      Drupal.AjaxCommands.prototype.
        PrivateMessageWindowsInboxUpdate = function (ajax, response) {
        // For jSlint compatibility.
        ajax = ajax;

        updateInbox(response.threadIds, response.newThreads);
      };

      Drupal.AjaxCommands.prototype.PrivateMessageWindowsFillThread = function (ajax, response) {
        // For jSlint compatibility.
        ajax = ajax;

        loadThreadResponse(response.threadId, response.lastMsgId, response.userName, response.userPicture, response.messages);
      };

      Drupal.AjaxCommands.prototype.PrivateMessageThreadMsgsUpdate = function (ajax, response) {
        // For jSlint compatibility.
        ajax = ajax;

        updateThreadMsgs(response.messages, response.threadId, response.lastMsgId);
      };
    },
    detach:function (context) {
    }
  };


}(jQuery, Drupal, drupalSettings, window));







