<?php

namespace Drupal\private_message_windows\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\private_message\Entity\PrivateMessage;
use Drupal\private_message\Service\PrivateMessageThreadManagerInterface;
use Drupal\private_message_windows\Ajax\PrivateMessageWindowsFillThreadMessagesCommand;
use Drupal\private_message_windows\Ajax\PrivateMessageWindowsInboxUpdateCommand;
use Drupal\private_message_windows\Ajax\PrivateMessageWindowsInsertOldMessagesCommand;
use Drupal\private_message\Entity\PrivateMessageThread;
use Drupal\private_message\Service\PrivateMessageServiceInterface;
use Drupal\private_message_windows\Ajax\PrivateMessageWindowsInsertNewMessagesCommand;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Controller to handle Ajax requests.
 */
class AjaxController extends ControllerBase{

  const AUTOCOMPLETE_COUNT = 10;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The entity manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The private message thread manager.
   *
   * @var \Drupal\Core\Entity\Sql\SqlContentEntityStorage
   */
  protected $threadManager;

  /**
   * The private message service.
   *
   * @var \Drupal\private_message\Service\PrivateMessageServiceInterface
   */
  protected $privateMessageService;

  /**
   * The private message service.
   *
   * @var \Drupal\private_message\Service\PrivateMessageThreadManagerInterface
   */
  protected $privateMessageThreadManager;

  /**
   * Constructs n AjaxController object.
   *
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory.
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   * @param \Drupal\private_message\Service\PrivateMessageServiceInterface $privateMessageService
   *   The private message service.
   * @param \Drupal\private_message\Service\PrivateMessageThreadManagerInterface $privateMessageThreadManager
   *   The private message service.
   */
  public function __construct(
    RendererInterface $renderer,
    RequestStack $requestStack,
    EntityTypeManagerInterface $entityTypeManager,
    ConfigFactoryInterface $configFactory,
    AccountProxyInterface $currentUser,
    PrivateMessageServiceInterface $privateMessageService,
    PrivateMessageThreadManagerInterface $privateMessageThreadManager
  ) {
    $this->renderer = $renderer;
    $this->requestStack = $requestStack;
    $this->entityTypeManager = $entityTypeManager;
    $this->threadManager = $entityTypeManager->getStorage('private_message_thread');
    $this->configFactory = $configFactory;
    $this->currentUser = $currentUser;
    $this->privateMessageService = $privateMessageService;
    $this->privateMessageThreadManager = $privateMessageThreadManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('renderer'),
      $container->get('request_stack'),
      $container->get('entity_type.manager'),
      $container->get('config.factory'),
      $container->get('current_user'),
      $container->get('private_message.service'),
      $container->get('private_message.thread_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function ajaxCallback($op) {
    $response = new AjaxResponse();

    if ($this->currentUser->hasPermission('use private messaging system')) {
      switch ($op) {

        case 'get_new_inbox_threads':
          $this->getNewInboxThreads($response);
          break;
        case 'get_old_inbox_threads':
          $this->getOldInboxThreads($response);
          break;
        case 'load_thread':
          $this->loadThread($response);
          break;
        case 'get_new_thread_msgs':
          $this->getNewPrivateMessages($response);
          break;
        case 'get_old_thread_msgs':
          $this->getOldPrivateMessages($response);
          break;
        case 'send_message':
          $this->sendMessage($response);
          break;
        case 'new_thread':
          $this->newThread($response);
          break;
      }
    }

    return $response;
  }


  /**
   * Creates an Ajax Command with new threads for the private message inbox.
   *
   * @param Drupal\Core\Ajax\AjaxResponse $response
   *   The response to which any commands should be attached.
   */
  protected function getNewInboxThreads(AjaxResponse $response) {
    $ids = $this->requestStack->getCurrentRequest()->get('ids');

    // Check to see if any thread IDs were POSTed.
    if (is_array($ids) and count($ids)) {
      // Get new inbox information based on the posted IDs.
      $inbox_threads = $this->privateMessageService->getUpdatedInboxThreads($ids);
    } else {
      // No IDs were posted, so the maximum possible number of threads to be
      // returned is retrieved from the block settings.
      $thread_count = $this->configFactory->get('block.block.privatemessageinbox')->get('settings.thread_count');
      $inbox_threads = $this->privateMessageService->getUpdatedInboxThreads([], $thread_count);
    }

    $threadList = private_message_windows_get_last_threads($inbox_threads['new_threads'], $this->currentUser);

    // Only need to do something if any thread IDS were found.
    if (count($threadList)) {
      //$view_builder = $this->entityTypeManager->getViewBuilder('private_message_thread');

      // Render any new threads as HTML to be sent to the browser.
      $rendered_threads = [];
      //foreach (array_keys($inbox_threads['new_threads']) as $thread_id) {
      foreach ($threadList as $thread_id => $renderable_thread) {
        //$renderable_thread = $view_builder->view($inbox_threads['new_threads'][$thread_id], 'inbox');
        $rendered_threads[$thread_id] = $this->renderer->renderRoot($renderable_thread);
      }

      // Add the command that will tell the inbox which thread items to update.
      $response->addCommand(new PrivateMessageWindowsInboxUpdateCommand(array_keys($threadList), $rendered_threads));
    }
  }

  /**
   * @param $messages \Drupal\private_message\Entity\PrivateMessage[]
   */
  protected function getRenderableMessages($messages) {
    $viewBuilder = $this->entityTypeManager->getViewBuilder('private_message');
    $renderable = [];
    foreach ($messages as $message) {
      if (!$message->access('view', $this->currentUser)) continue;
      $renderable[] = $viewBuilder->view($message, 'windows_inbox');
    }
    return $renderable;
  }


  /**
   * Load a private message thread to be dynamically inserted into the page.
   *
   * @param Drupal\Core\Ajax\AjaxResponse $response
   *   The response to which any commands should be attached.
   */
  protected function loadThread(AjaxResponse $response) {
    $thread_id = $this->requestStack->getCurrentRequest()->get('threadid');
    if ($thread_id) {
      $thread = PrivateMessageThread::load($thread_id);

      $this->runFillThread($response, $thread);

    }
  }

  /**
   * Filling Thread data and run command to draw it
   *
   * @param AjaxResponse $response
   * @param PrivateMessageThread $thread
   */
  protected function runFillThread(AjaxResponse $response, $thread) {
    if (!$thread or !$thread->access('view', $this->currentUser)) return;

    $this->privateMessageService->updateLastCheckTime();

    $messages = $thread->getMessages();
    $renderable = $this->getRenderableMessages($messages);
    $rendered_thread = $this->renderer->renderRoot($renderable);

    $lastMessage = array_pop($messages);
    $lastMsgId = $lastMessage? $lastMessage->id() : 0;

    $users = $thread->getMembers();
    $userPicture = $userName = '';
    foreach ($users as $tuser) {
      if($tuser->id() == $this->currentUser->id()) continue;
      $userPicture = private_message_windows_get_user_thumbnail($tuser);
      $fName = $tuser->get('field_name')->getValue();
      $userName = $fName[0]['value'] ?? '';
    }

    //unset($renderable["#attached"]["library"]);
    //$response->setAttachments($renderable['#attached']);

    $response->addCommand(new PrivateMessageWindowsFillThreadMessagesCommand($rendered_thread,
          $thread->id(), $lastMsgId, $userName, $userPicture));

  }


  /**
   * Creates an Ajax Command containing new private message.
   *
   * @param Drupal\Core\Ajax\AjaxResponse $response
   *   The response to which any commands should be attached.
   */
  protected function getNewPrivateMessages(AjaxResponse $response) {
    $thread_id = $this->requestStack->getCurrentRequest()->get('thread_id');
    $lastMessageId = $this->requestStack->getCurrentRequest()->get('message_id');

    if (!is_numeric($thread_id) or !is_numeric($lastMessageId)) return;

    $thread = $this->threadManager->load($thread_id);
    if (!$thread) return;

    $this->getNewPrivateMessagesAction($response, $thread, $lastMessageId);
  }


  protected function getNewPrivateMessagesAction($response, $thread, $lastMessageId) {
    $new_messages = $this->privateMessageService->getNewMessages($thread->id(), $lastMessageId);
    $this->privateMessageService->updateThreadAccessTime($thread);
    $renderable = [];

    $lastMessageId = 0;
    if (count($new_messages)) {
      $renderable = $this->getRenderableMessages($new_messages);
      $lastMessage = array_pop($new_messages);
      $lastMessageId = $lastMessage->id();

      // Ensure the browser knows the thread ID at all times.
      $renderable['#attached']['drupalSettings']['privateMessageThread']['threadId'] = (int) $thread->id();
    }

    $response->addCommand(new PrivateMessageWindowsInsertNewMessagesCommand($this->renderer->renderRoot($renderable), (int) $thread->id(), $lastMessageId));

  }


  /**
   * Create an Ajax Command containing old private messages threads.
   *
   * @param Drupal\Core\Ajax\AjaxResponse $response
   *   The response to which any commands should be attached.
   */
  protected function getOldInboxThreads(AjaxResponse $response) {

  }


  /**
   * Add new message and create ajax-command to add new messages to the thread
   *
   * @param Drupal\Core\Ajax\AjaxResponse $response
   *   The response to which any commands should be attached.
   */
  protected function sendMessage(AjaxResponse $response) {
    $current_request = $this->requestStack->getCurrentRequest();
    $thread_id = $current_request->get('thread_id');
    $text = $current_request->get('text');
    $lastMessageId = $current_request->get('last_message_id');

    if(!is_numeric($thread_id)) return;
    if(!is_numeric($lastMessageId)) return;
    if(!$text) return;

    /** @var PrivateMessageThread $thread */
    $thread = $this->threadManager->load($thread_id);
    if (!$thread) return;

    $privateMessage = PrivateMessage::create();
    $privateMessage->message = $text;
    $privateMessage->setOwnerId($this->currentUser->id());
    $privateMessage->save();

    //$thread->get('private_messages')->appendItem($privateMessage->id());
    $thread->addMessage($privateMessage);
    $thread->save();

    $this->getNewPrivateMessagesAction($response, $thread, $lastMessageId);

  }


  /**
   * Find thread by recipient id or create new thread for user
   *
   * @param Drupal\Core\Ajax\AjaxResponse $response
   *   The response to which any commands should be attached.
   */
  protected function newThread(AjaxResponse $response) {
    $current_request = $this->requestStack->getCurrentRequest();
    $uid = (int)$current_request->get('recip_id');
    if(!$uid) return;

    $thread = $this->privateMessageService->getThreadForMembers([$this->currentUser, User::load($uid)]);
    $this->runFillThread($response, $thread);
  }


  /**
   * Create an Ajax Command containing old private messages.
   *
   * @param Drupal\Core\Ajax\AjaxResponse $response
   *   The response to which any commands should be attached.
   */
  protected function getOldPrivateMessages(AjaxResponse $response) {
    $current_request = $this->requestStack->getCurrentRequest();
    $thread_id = $current_request->get('thread_id');
    $message_id = $current_request->get('messageid');
    if (!is_numeric($thread_id) or is_numeric($message_id)) return;

    $message_info = $this->privateMessageService->getPreviousMessages($thread_id, $message_id);

    if (count($message_info['messages'])) {
      $messages = [];
      $view_builder = $this->entityTypeManager->getViewBuilder('private_message');
      $has_next = $message_info['next_exists'];
      foreach ($message_info['messages'] as $message) {
        if ($message->access('view', $this->currentUser)) {
          $messages[] = $view_builder->view($message);
        }
      }

      $response->addCommand(new PrivateMessageWindowsInsertOldMessagesCommand($this->renderer->renderRoot($messages), $thread_id, count($message_info['messages']), $has_next));
    } else {
      $response->addCommand(new PrivateMessageWindowsInsertOldMessagesCommand('', $thread_id, 0, FALSE));
    }
  }

}
